@@ -2,32 +2,35 @@
 # Estimate forecast error variance
 #  source code adapted from the MTS package
 #---------------------------------------------
-fevd = function(Phi,  Sig, lag = 4)
+fevd = function(Phi,  Sig, lag = 4, structure = NA)
 {
   if (length(Phi) > 0) {
     if (!is.matrix(Phi))
       Phi = as.matrix(Phi)
   }
 
-  if (!is.matrix(Sig))
-    Sig = as.matrix(Sig)
   if (lag < 1)
     lag = 1
+
   p = 0
+
   if (length(Phi) > 0) {
     k = nrow(Phi)
     m = ncol(Phi)
     p = floor(m/k)
   }
+
   q = 0
 
   Si = diag(rep(1, k))
 
   m = (lag + 1) * k
   m1 = (q + 1) * k
+
   if (m > m1) {
     Si = cbind(Si, matrix(0, k, (m - m1)))
   }
+
   if (p > 0) {
     for (i in 1:lag) {
       if (i <= p) {
@@ -52,39 +55,47 @@ fevd = function(Phi,  Sig, lag = 4)
       Si[, (kdx + 1):(kdx + k)] = tmp
     }
   }
+
   orSi = NULL
-  m1 = chol(Sig)
+  m1 = Sig
   P = t(m1)
   orSi = P
+
   for (i in 1:lag) {
     idx = i * k
     w1 = Si[, (idx + 1):(idx + k)]
     w2 = w1 %*% P
     orSi = cbind(orSi, w2)
   }
+
   orSi2 = orSi^2
   Ome = orSi2[, 1:k]
   wk = Ome
+
   for (i in 1:lag) {
     idx = i * k
     wk = wk + orSi2[, (idx + 1):(idx + k)]
     Ome = cbind(Ome, wk)
   }
+
   FeV = NULL
   OmeRa = Ome[, 1:k]
   FeV = cbind(FeV, apply(OmeRa, 1, sum))
   OmeRa = OmeRa/FeV[, 1]
+
   for (i in 1:lag) {
     idx = i * k
     wk = Ome[, (idx + 1):(idx + k)]
     FeV = cbind(FeV, apply(wk, 1, sum))
     OmeRa = cbind(OmeRa, wk/FeV[, (i + 1)])
   }
+
   for (i in 1:(lag + 1)) {
     idx = (i - 1) * k
     Ratio = OmeRa[, (idx + 1):(idx + k)]
   }
-  FEVdec <- list(irf = Si, orthirf = orSi, Omega = Ome, OmegaR = OmeRa)
+
+  FEVdec = list(irf = Si, orthirf = orSi,  Omega = Ome, OmegaR = OmeRa)
 
   return(FEVdec)
 }
@@ -94,6 +105,9 @@ fevd = function(Phi,  Sig, lag = 4)
 #--------------------------------------------------------
 #' Estimate forecast error variance decomposition
 #'
+#' Estimate forecast error variance decomposition for VARs
+#' with either short or 'IV-short' structural errors.
+#'
 #' @param var              VAR output
 #' @param horizon          int: number of periods
 #' @param scale            boolean: scale variable contribution as percent of total error
@@ -133,7 +147,7 @@ fevd = function(Phi,  Sig, lag = 4)
 #'
 #' # forecast error variance decomposition
 #' var.fevd = sovereign::var_fevd(var)
-#' 
+#'
 #' # historical shock decomposition
 #' var.hd = sovereign::var_hd(var)
 #'
@@ -151,6 +165,12 @@ var_fevd = function(
   if(!is.numeric(horizon) | horizon %% 1 != 0 | horizon <= 0){
     stop('horizon must be a positive integer')
   }
+  if(class(var) != 'VAR'){
+    stop('var must be a VAR object')
+  }
+  if(!var$structure %in% c('short', 'IV-short')){
+    stop('FEVD method is only available for VARs with short or IV-short structural errors')
+  }
 
   # function variables
   forecast.date = y = shock = error = NULL
@@ -162,12 +182,14 @@ var_fevd = function(
   regressors = colnames(dplyr::select(data, -date))
 
   # error covariance matrix
-  cov.matrix = stats::var(stats::na.omit(dplyr::select(residuals, -date, -forecast.date)))
+  B = solve_B(var)
+  B = as.matrix(B)
 
   # forecast error variance decomposition
-  errors = fevd(Phi =  dplyr::select(coef, -y, -dplyr::contains('cosnt'), -dplyr::contains('trend')),
-                Sig = cov.matrix,
-                lag = horizon)
+  errors = fevd(Phi =  dplyr::select(coef, -y, -dplyr::contains('const'), -dplyr::contains('trend')),
+                Sig = B,
+                lag = horizon,
+                structure = var$structure)
 
   # reorganize results
   response = data.frame(t(errors$OmegaR))
@@ -178,7 +200,15 @@ var_fevd = function(
 
   # cast to long-form
   response = response %>%
-    tidyr::pivot_longer(cols = regressors, names_to = 'response', values_to =  'error')
+    tidyr::pivot_longer(!c('shock','horizon'), names_to = 'response', values_to = 'error')
+
+
+  if(var$structure %in% c('IV-short')){
+    labels = c(var$instrumented, regressors[!regressors %in% var$instrumented])
+    labels = rep(labels, length(labels) * (horizon+1))
+    response$response = labels
+    response$shock = labels[order(labels)]
+  }
 
   # scale responses
   if(scale == TRUE){
@@ -193,6 +223,9 @@ var_fevd = function(
 
 #' Estimate regime-dependent forecast error variance decomposition
 #'
+#' Estimate forecast error variance decomposition for RVARs
+#' with either short or 'IV-short' structural errors.
+#'
 #' @param rvar             RVAR output
 #' @param horizon          int: number of periods
 #' @param scale            boolean: scale variable contribution as percent of total error
@@ -235,7 +268,7 @@ var_fevd = function(
 #'
 #' # forecast error variance decomposition
 #' rvar.fevd = sovereign::rvar_fevd(rvar)
-#' 
+#'
 #' # historical shock decomposition
 #' rvar.hd = sovereign::rvar_hd(rvar)
 #'
@@ -254,6 +287,12 @@ rvar_fevd = function(
   if(!is.numeric(horizon) | horizon %% 1 != 0 | horizon <= 0){
     stop('horizon must be a positive integer')
   }
+  if(class(rvar) != 'RVAR'){
+    stop('rvar must be a RVAR object')
+  }
+  if(!rvar$structure %in% c('short', 'IV-short')){
+    stop('FEVD method is only available for VARs with short or IV-short structural errors')
+  }
 
   # function variables
   forecast.date = y = shock = error = model.regime = NULL
@@ -274,16 +313,38 @@ rvar_fevd = function(
 
       # set regime specific data
       coef = rvar$model[[paste0('regime_',regime.val)]]$coef
+
       residuals = rvar$residuals[[1]] %>%
         dplyr::filter(model.regime == regime.val)
 
-      # error covariance matrix
-      cov.matrix = stats::var(stats::na.omit(dplyr::select(residuals, -date, -model.regime, -forecast.date)))
+      # instrument
+      if(!is.null(rvar$instrument)){
+        instrument = dplyr::inner_join(rvar$instrument, dplyr::select(residuals, date), by = 'date')
+      }else{
+        instrument = NULL
+      }
+
+      # regime-specific model
+      model =  rvar$model[[paste0('regime_',regime.val)]]
+
+      # structural error variance-covariance matrix
+      B =
+        solve_B(
+          var = list(
+            model = model,
+            residuals = residuals %>%
+              dplyr::select(-model.regime),
+            structure = rvar$structure,
+            instrument = instrument,
+            instrumented = rvar$instrumented
+          )
+        )
 
       # forecast error variance decomposition
-      errors = fevd(Phi =  dplyr::select(coef, -y, -dplyr::contains('cosnt'), -dplyr::contains('trend')),
-                    Sig = cov.matrix,
-                    lag = horizon)
+      errors = fevd(Phi =  dplyr::select(coef, -y, -dplyr::contains('const'), -dplyr::contains('trend')),
+                    Sig = B,
+                    lag = horizon,
+                    structure = rvar$structure)
 
       # reorganize results
       response = data.frame(t(errors$OmegaR))
@@ -294,7 +355,15 @@ rvar_fevd = function(
 
       # cast to long-form
       response = response %>%
-        tidyr::pivot_longer(cols = regressors, names_to = 'response', values_to =  'error')
+        tidyr::pivot_longer(!c('shock','horizon'), names_to = 'response', values_to =  'error')
+
+      # correct labels for IV-short
+      if(rvar$structure %in% c('IV-short')){
+        labels = c(rvar$instrumented, regressors[!regressors %in% rvar$instrumented])
+        labels = rep(labels, length(labels) * (horizon+1))
+        response$response = labels
+        response$shock = labels[order(labels)]
+      }
 
       # scale responses
       if(scale == TRUE){
